
package com.embeddedartists.romfs;

/*
 * $Id: Tree.java,v 1.1.1.1 2005/08/19 08:57:06 rebert Exp $
 *	Collect files in a tree structure.
 */

import java.io.*;



/**
 * Tree
 * 
 */
public class Tree {


  /**
   * Construct a filetree object based on the contents of the given
   * directory
   *
   * @param pathName the root of the tree
   */
  Tree(String pathName) {
    file = new File(pathName);

    if (!file.exists()) {
      System.err.println("Error: \"" + pathName + "\" does not exist");
      System.exit(1);
    }

    if (file.isDirectory()) {
      if (file.getName().toLowerCase().equals(".svn")) {
        // no point in adding subersion folders
        System.out.println("Skipping Subversion folder \"" + pathName + "\"");
        children = new Tree[0];
      }
      else {
        String    contents[];

        contents = file.list();
        children = new Tree[contents.length];

        for (int i = 0; i < contents.length; i++) {
          children[i] = new Tree(pathName +
                                 File.separatorChar +
                                 contents[i]);
        }
      }
    }
    else {
      children = null;
    }
  }

  /**
   * Test if this node is a directory
   *
   * @return true if directory; otherwise false if file
   */
  public boolean isDirectory() {
    return file.isDirectory();
  }


  /**
   * Test if this node is a file
   *
   * @return true if file; otherwise false
   */
  public boolean isFile() {
    return file.isFile();
  }

  /**
   * Fetch the name of this node
   *
   * @return the name of this node
   */
  public String getName() {
    return file.getName();
  }

  /**
   * Fetch the children of this node.
   *
   * @return the children of this node
   */
  public Tree[] getChildren() {
    return children;
  }

  /**
   * Get the size of the file
   *
   */
  public int getFileSize() {
    return(int)file.length();
  }

  /**
   * Fetch the file contents
   *
   * @return the file contents
   */
  public byte[] getFileContents()
  {
    byte data[] = new byte[(int)file.length()];
    FileInputStream fin = null;

    try {
      fin = new FileInputStream(file);
    }
    catch (FileNotFoundException err) {
      System.err.println("Could not open " + file.getName());
      System.exit(1);
    }

    try {
      fin.read(data);
    }
    catch (IOException err) {
      System.err.println("Could not read " + file.getName());
      System.exit(1);
    }

    return data;
  }


  File      file;       // Info about this node
  Tree      children[];   // All child nodes.
}
