/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __APPLICATION_H
#define __APPLICATION_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

typedef struct
{
  void (*start)(void);
  void (*stop)(void);
  void (*timeout)(void);

  uint32_t intervalInMs;

  // internal fields, don't modify these
  Bool stopped;
} AppCallback_t;

typedef enum {
  APP_PRIO_1 = 3,
  APP_PRIO_2 = 4,
  APP_PRIO_3 = 5
} AppPrio_t;

typedef struct
{
  AppPrio_t prio;
  AppCallback_t* (*init)(void);
  const char* pName;
  Bool autoStart;
} AppTableEntry_t;


/******************************************************************************
 * Public variables
 *****************************************************************************/

extern volatile uint32_t msClock;

/******************************************************************************
 * Public functions
 *****************************************************************************/


/*****************************************************************************
 *
 * Description:
 *    Executes the first App in the "App Table" (found in appTable.h) marked as
 *    autoStart=TRUE.
 *
 *    This function is called from the main() function to start the default
 *    App, which is normally the menu system.
 *
 * Params:
 *    None
 *
 * Return:
 *    None
 *
 ****************************************************************************/
void appStartInitial(void);


/*****************************************************************************
 *
 * Description:
 *    Stops the currently running App.
 *
 * Params:
 *    None
 *
 * Return:
 *    None
 *
 ****************************************************************************/
void appClose(void);


/*****************************************************************************
 *
 * Description:
 *    Starts the App with the specified name.
 *
 * Params:
 *    [in] pName   - Name of the App to start
 *
 * Return:
 *    TRUE if the App was started
 *
 ****************************************************************************/
Bool appStartApp(const char* pName);


/*****************************************************************************
 *
 * Description:
 *    Starts the App with the specified id (i.e. position in the "App Table"
 *    in apptable.h).
 *
 * Params:
 *    [in] id   - ID of the App to start
 *
 * Return:
 *    TRUE if the App was started
 *
 ****************************************************************************/
Bool appStartAppID(int id);


/*****************************************************************************
 *
 * Description:
 *    Returns the name of the App with the specified id (i.e. position in
 *    the "App Table" in apptable.h).
 *
 *    Note that the returned string is not allocated and hence should not
 *    be freed.
 *
 * Params:
 *    [in] id   - ID of the App to get the name for
 *
 * Return:
 *    The name of the App, or NULL if the App could not be found.
 *
 ****************************************************************************/
const char* appGetAppName(int id);

void appRebootAfterDelay(int delayInMs);

#endif /* end __APPLICATION_H */
