/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __LOG_H
#define __LOG_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <stdio.h>
#include "lpc_types.h"
#include "board.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define dbg(...) do {\
    sprintf((char*)g_dbbuf, __VA_ARGS__); \
    console_sendString(g_dbbuf); \
  } while(0)

#define log_i(...) do {\
    sprintf((char*)g_dbbuf, __VA_ARGS__); \
    console_sendString(g_dbbuf); \
  } while(0)

#define log_e(...) do {\
    sprintf((char*)g_dbbuf, __VA_ARGS__); \
    console_sendString((uint8_t*)"SYS_ERROR: "); \
    console_sendString(g_dbbuf); \
  } while(0)

#define LED_On(x)   do { \
    g_LED_buf |= (x); \
    *g_LED_reg = g_LED_buf; \
  } while(0)

#define LED_Off(x)  do { \
    g_LED_buf &= ~(x); \
    *g_LED_reg = g_LED_buf; \
  } while(0)

#define LED_Toggle(x)  do { \
    g_LED_buf ^= (x); \
    *g_LED_reg = g_LED_buf; \
  } while(0)


#define LED_FRAME_OUT   (1 << 0)
#define LED_DEC_SLOWER  0//(1 << 1)
#define LED_DEC_FASTER  (1 << 2)
#define LED_IRQ         (1 << 3)
#define LED_SEND        (1 << 4)
#define LED_IN_WHILE    0//(1 << 5)
#define LED_IN_DECODE   (1 << 6)
#define LED_FS_READ     (1 << 7)
#define LED_VU_DRAW     (1 << 1)
#define LED_VU_PROCESS  (1 << 5)
#define LED_GUI_TOUCH   (1 << 8)
#define LED_TEST        0//(1 << 8)


/******************************************************************************
 * Global Variables
 *****************************************************************************/

extern uint8_t g_dbbuf[];

extern volatile uint16_t* g_LED_reg;
extern volatile uint16_t  g_LED_buf;

#endif /* end __LOG_H */

