/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __ROM_FSSTUB_H
#define __ROM_FSSTUB_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <string.h>
#include <stdlib.h>

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Take, lock, a synchronization primitive. This macro is used to protect
 *    the file system when it is used in a multi-threaded environment.
 *
 * Prototype
 *    void m_lock_fs
 *
 *****************************************************************************/
#define m_lock_fs

/******************************************************************************
 *
 * Description:
 *    Give, unlock, a synchronization primitive previously locked by m_lock_fs
 *
 * Prototype:
 *    void m_unlock_fs
 *
 *****************************************************************************/
#define m_unlock_fs

/******************************************************************************
 *
 * Description:
 *    Copy a byte array.
 *
 * Params:
 *    [out] pDest - The destination address.
 *    [in]  pSrc  - The source address.
 *    [in]  len   - How many bytes to copy.
 *
 *****************************************************************************/
#define m_fs_memcpy(pDest, pSrc, len) memcpy(pDest, pSrc, len)

/******************************************************************************
 *
 * Description:
 *    Locate the first occurence of a given character in a string.
 *
 * Params:
 *    [in] pStr - The string to search.
 *    [in] ch   - The character to find.
 *
 * Returns:
 *    The address of the found character or NULL.
 *
 *****************************************************************************/
#define m_fs_strchr(pStr, ch) strchr(pStr, ch)

/******************************************************************************
 *
 * Description:
 *    Calculate the string length.
 *
 * Params:
 *    [in] pStr - The string.
 *
 * Returns:
 *    The string length.
 *
 *****************************************************************************/
#define m_fs_strlen(pStr) strlen(pStr)


#endif /* end __ROM_FSSTUB_H */
