/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __TRANS_FS_H
#define __TRANS_FS_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define FATFS_PATH_PREFIX ""  //"/ea_demo"

typedef enum
{
  ROMFS_OVERRIDES_FATFS,
  FATFS_OVERRIDES_ROMFS,
  ONLY_USE_ROMFS,
  ONLY_USE_FATFS
} tFsOverride;

typedef enum
{
  TFS_FS_ROM,
  TFS_FS_FAT
} tFsFileSystem;

typedef enum
{
  TFS_OK,
  TFS_ERR_NO_SUCH_FILE,
  TFS_ERR_INVALID_INPUT,
  TFS_ERR_INVALID_SIZE,
  TFS_ERR_MALLOC_FAILED,
  TFS_ERR_INVALID_OVERRIDE,
  TFS_ERR_NO_SUCH_FS,
  TFS_ERR_FS_IS_READONLY,
  TFS_ERR
} tTransFsError;


/******************************************************************************
 * Public Functions
 *****************************************************************************/


/*****************************************************************************
 *
 * Description:
 *    The function will check if a MMC card is present and initialize that
 *    file system if needed.
 *
 * Params:
 *    None
 *
 * Return:
 *    TFS_OK if a MMC card is present and initialized,
 *    otherwise one of the TFS_ERR_* error codes.
 *
 ****************************************************************************/
tTransFsError transfs_initializeFATFS(void);


/*****************************************************************************
 *
 * Description:
 *    Allocates a new buffer and loads the specified file into that buffer.
 *    The value of the behavior parameter determines the search order when
 *    locating the file.
 *
 * Params:
 *    [in]  pFileName  - the name of the file to locate.
 *    [in]  behavior   - what should be done if file is found i multiple locations.
 *    [out] ppDest     - will point to newly allocated memory with the file.
 *    [out] pSize      - the file size and number of bytes stored in ppDest.
 *    [out] pFsType    - where was the file located.
 *
 * Return:
 *    TFS_OK if the file was located and its contents stored in ppDest,
 *    otherwise one of the TFS_ERR_* error codes.
 *
 ****************************************************************************/
tTransFsError transfs_loadFileIntoNewBuff(char* pFileName, tFsOverride behavior,
    uint8_t** ppDest, uint32_t* pSize, tFsFileSystem* pFsType);

/*****************************************************************************
 *
 * Description:
 *    Creates a new file (possibly overwriting an old one) and writes the
 *    buffer data into it.
 *
 * Params:
 *    [in] pFileName  - the name of the file to write to.
 *    [in] fsType     - which file system to store the file in
 *    [in] pBuff      - the data to store
 *    [in] buffSize   - the number of bytes in buff
 *
 * Return:
 *    TFS_OK if the file was created and all bytes written,
 *    otherwise one of the TFS_ERR_* error codes.
 *
 ****************************************************************************/
tTransFsError transfs_storeBuffInFile(char* pFileName, tFsFileSystem fsType,
    uint8_t* pBuff, uint32_t buffSize);

#endif /* end __TRANS_FS_H */

