#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "lpc177x_8x_timer.h"
#include "lpc177x_8x_i2c.h"
#include "board.h"
#include "sdram.h"

#include "ea_lcd_board.h"
#include "lpc_swim.h"

#include "application.h"
#include "globals.h"
#include "swim_extensions.h"
#include "lpc_heap.h"

// Number of points to draw
#define N 256

#define PI 3.1415926535897932384626433832795


// for draw()
static uint16_t* oldProjX = NULL;
static uint16_t* oldProjY = NULL;
static uint8_t* oldDotSize = NULL;

static SWIM_WINDOW_T win1;
static int32_t windowX;
static int32_t windowY;

void matrix(float xyz[3][N], uint8_t rgb[3][N]) {
              float increment = 2 / sqrt(N);
 static const float PI2 = 2 * PI;
 static const float PI23 = 2 * PI / 3;
 static const float PI43 = 4 * PI / 3;
              float incrementT = PI2 / 50;
 static float t = 0;
 uint16_t i;

 float x = -1;
 float y = -1;
 for(i = 0; i < N; i++) {

  xyz[0][i] = x;
  xyz[1][i] = y;
  xyz[2][i] = sin(PI2 * sin(t / 20) * sqrt(x * x + y * y) + t) / 2;

  rgb[0][i] = floor(cos(xyz[2][i] * PI2) * RED_COLORS / 2 + (RED_COLORS / 2 - 1));
  rgb[1][i] = floor(cos(xyz[2][i] * PI2 + PI23) * GREEN_COLORS / 2 + (GREEN_COLORS / 2 - 1));
  rgb[2][i] = floor(cos(xyz[2][i] * PI2 + PI43) * BLUE_COLORS / 2 + (BLUE_COLORS / 2 - 1));

  x += increment;
  if(x >= 1) {
   x = -1;
   y += increment;
  }

 }
 t += incrementT;
}

void rotate(float xyz[3][N], uint8_t rgb[3][N],
            float angleX, float angleY, float angleZ) {
 float tmpX;
 float tmpY;
 float sinx = sin(angleX);
 float cosx = cos(angleX);
 float siny = sin(angleY);
 float cosy = cos(angleY);
 float sinz = sin(angleZ);
 float cosz = cos(angleZ);
 uint16_t i;

 for(i = 0; i < N; i++) {
  tmpX      = xyz[0][i] * cosx - xyz[2][i] * sinx;
  xyz[2][i] = xyz[0][i] * sinx + xyz[2][i] * cosx;
  xyz[0][i] = tmpX;

  tmpY      = xyz[1][i] * cosy - xyz[2][i] * siny;
  xyz[2][i] = xyz[1][i] * siny + xyz[2][i] * cosy;
  xyz[1][i] = tmpY;

  tmpX      = xyz[0][i] * cosz - xyz[1][i] * sinz;
  xyz[1][i] = xyz[0][i] * sinz + xyz[1][i] * cosz;
  xyz[0][i] = tmpX;
 }
}

void draw(float xyz[3][N], uint8_t rgb[3][N]) {
 uint16_t projX;
 uint16_t projY;
 uint8_t dotSize;
 uint16_t i;

 for(i = 0; i < N; i++) {
  projX = windowX / 2 + floor(xyz[0][i] * (250 - 50 * (xyz[2][i] + 1)));
  projY = windowY / 2 + floor(xyz[1][i] * (250 - 50 * (xyz[2][i] + 1)));
  dotSize = floor(4 - 2 * (xyz[2][i] + 1));

  swim_set_pen_color(&win1, 0);
  swimex_put_circle(&win1, oldProjX[i], oldProjY[i], oldDotSize[i], 1);

  if(projX > 0 && projY > 0 &&
     projX < windowX - dotSize && projY < windowY - dotSize) {
   swim_set_pen_color(&win1, (rgb[0][i] << REDSHIFT) + (rgb[1][i] << GREENSHIFT) + (rgb[2][i] << BLUESHIFT));
   swimex_put_circle(&win1, projX, projY, dotSize, 1);
   oldProjX[i] = projX;
   oldProjY[i] = projY;
   oldDotSize[i] = dotSize;
  }
 }
}

static void* allocAndClear(uint32_t size)
{
	void* ptr = lpc_new(size);
	memset(ptr, 0, size);
	return ptr;
}

static void show3DGrid(void) 
{
  float (*xyz)[N]   = allocAndClear(sizeof(float)*3*N);   // float xyz[3][N]
  uint8_t (*rgb)[N] = allocAndClear(sizeof(uint8_t)*3*N); // float rgb[3][N]
	oldProjX          = allocAndClear(sizeof(uint16_t)*N);  // uint16_t[N]
	oldProjY          = allocAndClear(sizeof(uint16_t)*N);  // uint16_t[N]
	oldDotSize        = allocAndClear(sizeof(uint8_t)*N);   // uint8_t[N]

	windowX = GlobalParameters.display.width;
  windowY = GlobalParameters.display.height;

  // Create a SWIM window
  swim_window_open(&win1, windowX, windowY, (COLOR_T*)GlobalParameters.display.framebuffer, 0, 0,
                          windowX - 1, windowY - 1, 1, BLACK, BLACK, BLACK);

  while(1) {
   static float angleX = 0;
   static float angleY = 0;
   static float angleZ = 0;

   matrix(xyz, rgb);

   rotate(xyz, rgb, angleX, angleY, angleZ);

   draw(xyz, rgb);

   angleX += 0.003f;
   angleY += 0.002f;
   angleZ += 0.001f;
  }
}

static void cleanup(void)
{
	// Mandatory function, but it will never be called as the application
	// runs forever
}

/******************************************************************************
 * Implementation of public functions
 *****************************************************************************/

static AppCallback_t grid3dCallback = {
    &show3DGrid,
    &cleanup,
    NULL,
    0,
    FALSE,
};

AppCallback_t* grid3dAppInit(void)
{
  return &grid3dCallback;
}
