/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __STATEMACHINE_H
#define __STATEMACHINE_H

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "lpc_types.h"
#include "ff.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/** Max number of entries in the MP3 playlist */
#define MP3_PLAYLIST_MAX_LEN 255

typedef enum 
{
  EVENT_NOTHING,
  EVENT_CURSOR_UP,
  //EVENT_CURSOR_LEFT,
  EVENT_CURSOR_DOWN,
  //EVENT_CURSOR_RIGHT,
  EVENT_CURSOR_SET,
  EVENT_CURSOR_CLICK,
  EVENT_PLAY,   // == EVENT_CURSOR_CLICK
  EVENT_NEXT,   // == EVENT_CURSOR_DOWN + EVENT_CURSOR_CLICK
  EVENT_PREV,   // == EVENT_CURSOR_UP + EVENT_CURSOR_CLICK
  EVENT_PAUSE,

  EVENT_STOP,  
  EVENT_MUTE_TOGGLE,
} event_t;

typedef enum
{
  STATE_IDLE,
  STATE_PREPARING,
  STATE_PLAYING,
  STATE_PAUSED,
  STATE_STOPPING,
  STATE_STOPPING_FOR_NEXT,
} state_t;

typedef enum
{
  PLAYBACK_ERROR_OK,
  PLAYBACK_ERROR_FOPEN,
  PLAYBACK_ERROR_DECODER_FAILED,
  PLAYBACK_ERROR_UNSUPPORTED_MP3,
  PLAYBACK_ERROR_OUT_OF_DATA,
  PLAYBACK_ERROR_DMA_EMPTY,
} playback_error_t;

typedef playback_error_t(*playbackFunc)(void);

typedef struct MP3List_t
{
  uint8_t drive;		/* drive */
  char fname[13];   /* mp3 file name */
	char* longname;
} MP3List_t;

/******************************************************************************
 * Global Variables
 *****************************************************************************/

extern state_t CurrentState;
extern FIL CurrentFile;
extern uint32_t CurrentMP3;
extern uint32_t PreviousMP3;
extern MP3List_t MP3List[MP3_PLAYLIST_MAX_LEN];
extern uint32_t mp3_cnt;
extern volatile uint32_t Timer;  /* Performance timer (1kHz increment) */
extern volatile int actualTime;


/******************************************************************************
 * Functions
 *****************************************************************************/

void statemachine_run(playbackFunc initFunc, 
										 playbackFunc playFunc, 
										 playbackFunc stopFunc,
										 playbackFunc pauseFunc, 
										 playbackFunc resumeFunc);
										 
void statemachine_systick(void);

#endif /* end __STATEMACHINE_H */

